"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lsp_editor_adapter_1 = require("lsp-editor-adapter");
const coreutils_1 = require("@jupyterlab/coreutils");
const codemirror_1 = require("./codemirror");
const converter_1 = require("../converter");
const free_tooltip_1 = require("../free_tooltip");
const widgets_1 = require("@phosphor/widgets");
const utils_1 = require("../utils");
class JupyterLabWidgetAdapter {
    constructor(app, rendermime_registry, invoke) {
        this.app = app;
        this.rendermime_registry = rendermime_registry;
        this.invoke_command = invoke;
    }
    get root_path() {
        // TODO: serverRoot may need to be included for Hub or Windows, requires testing.
        // let root = PageConfig.getOption('serverRoot');
        return coreutils_1.PathExt.dirname(this.document_path);
    }
    invoke_completer() {
        return this.app.commands.execute(this.invoke_command);
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('LSP: will connect using root path:', this.root_path, 'and language:', this.language);
            this.connection = new lsp_editor_adapter_1.LspWsConnection({
                serverUri: 'ws://localhost/' + this.language,
                languageId: this.language,
                // paths handling needs testing on Windows and with other language servers
                rootUri: 'file:///' + this.root_path,
                documentUri: 'file:///' + this.document_path,
                documentText: this.get_document_content.bind(this)
            }).connect(new WebSocket('ws://localhost:3000/' + this.language));
            // @ts-ignore
            this.connection.on('goTo', this.handle_jump.bind(this));
            // @ts-ignore
            yield utils_1.until_ready(() => this.connection.isConnected, -1, 150);
            console.log('LSP:', this.document_path, 'connected.');
        });
    }
    handle_jump(locations) {
        // TODO: implement selector for multiple locations
        //  (like when there are multiple definitions or usages)
        if (locations.length === 0) {
            console.log('No jump targets found');
            return;
        }
        console.log('Will jump to the first of suggested locations:', locations);
        let location = locations[0];
        let uri = decodeURI(location.uri);
        let current_uri = this.connection.getDocumentUri();
        let cm_position = converter_1.PositionConverter.lsp_to_cm(location.range.start);
        let editor_index = this.adapter.get_editor_index(cm_position);
        let transformed_position = this.adapter.transform(cm_position);
        let transformed_ce_position = converter_1.PositionConverter.cm_to_ce(transformed_position);
        console.log('Jumping to', transformed_position, 'in', editor_index, 'editor of', uri);
        if (uri == current_uri) {
            this.jumper.jump({
                token: {
                    offset: this.jumper.getOffset(transformed_ce_position, editor_index),
                    value: ''
                },
                index: editor_index
            });
            return;
        }
        if (uri.startsWith('file://')) {
            uri = uri.slice(7);
        }
        this.jumper.global_jump({
            // TODO: there are many files which are not symlinks
            uri: '.lsp_symlink/' + uri,
            editor_index: editor_index,
            line: transformed_ce_position.line,
            column: transformed_ce_position.column
        }, true);
    }
    create_adapter() {
        this.adapter = new codemirror_1.CodeMirrorAdapterExtension(this.connection, { quickSuggestionsDelay: 50 }, this.cm_editor, this.create_tooltip.bind(this), this.invoke_completer.bind(this));
        console.log('LSP: Adapter for', this.document_path, 'is ready.');
    }
    get_doc_position_from_context_menu() {
        // get the first node as it gives the most accurate approximation
        let leaf_node = this.app.contextMenuHitTest(() => true);
        let { left, top } = leaf_node.getBoundingClientRect();
        // @ts-ignore
        let event = this.app._contextMenuEvent;
        // if possible, use more accurate position from the actual event
        // (but this relies on an undocumented and unstable feature)
        if (event !== undefined) {
            left = event.clientX;
            top = event.clientY;
            event.stopPropagation();
        }
        return this.cm_editor.coordsChar({
            left: left,
            top: top
        }, 'window');
    }
    create_tooltip(markup, cm_editor, position) {
        const bundle = markup.kind === 'plaintext'
            ? { 'text/plain': markup.value }
            : { 'text/markdown': markup.value };
        const tooltip = new free_tooltip_1.FreeTooltip({
            anchor: this.widget.content,
            bundle: bundle,
            editor: this.find_ce_editor(cm_editor),
            rendermime: this.rendermime_registry,
            position: converter_1.PositionConverter.cm_to_ce(this.adapter.transform(position)),
            moveToLineEnd: false
        });
        widgets_1.Widget.attach(tooltip, document.body);
        return tooltip;
    }
}
exports.JupyterLabWidgetAdapter = JupyterLabWidgetAdapter;
