"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const jupyterlab_1 = require("./jupyterlab");
const notebook_mapper_1 = require("../notebook_mapper");
const utils_1 = require("../utils");
const completion_1 = require("../completion");
class NotebookAdapter extends jupyterlab_1.JupyterLabWidgetAdapter {
    constructor(editor_widget, jumper, app, completion_manager, rendermime_registry) {
        super(app, rendermime_registry, 'completer:invoke-notebook');
        this.widget = editor_widget;
        this.editor = editor_widget.content;
        this.completion_manager = completion_manager;
        this.jumper = jumper;
        this.init_once_ready().then();
    }
    is_ready() {
        return (this.widget.context.isReady &&
            this.widget.content.isVisible &&
            this.widget.content.widgets.length > 0 &&
            this.language != '');
    }
    get document_path() {
        return this.widget.context.path;
    }
    get language() {
        let language_metadata = this.widget.model.metadata.get('language_info');
        // @ts-ignore
        return language_metadata.name;
    }
    find_ce_editor(cm_editor) {
        return this.notebook_as_editor.cm_editor_to_ieditor.get(cm_editor);
    }
    init_once_ready() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('LSP: waiting for', this.document_path, 'to fully load');
            yield utils_1.until_ready(this.is_ready.bind(this), -1);
            console.log('LSP:', this.document_path, 'ready for connection');
            // TODO
            // this.widget.context.pathChanged
            this.notebook_as_editor = new notebook_mapper_1.NotebookAsSingleEditor(this.widget, 
            // TODO: find better, language agnostic solution (or at least verify that kernel is IPython, not plain Python)
            // temporary workaround: filter out IPython line magics and shell assignments if the language is python
            this.language == 'python' ? '^[%!].*' : '', 
            // the pass is used to silence the linters in places where the line would otherwise be filtered out
            this.language == 'python' ? 'pass' : ''
            // TODO: ideally we would allow to substitute code for magics, etc
            );
            this.connect();
            this.create_adapter();
            // refresh server held state after every change
            // note this may be changed soon: https://github.com/jupyterlab/jupyterlab/issues/5382#issuecomment-515643504
            this.widget.model.contentChanged.connect(this.refresh_lsp_notebook_image.bind(this));
            // and refresh it after the cell was activated, just to make sure that the first experience is ok
            this.widget.content.activeCellChanged.connect(this.refresh_lsp_notebook_image.bind(this));
            // register completion connectors on cells
            // see https://github.com/jupyterlab/jupyterlab/blob/c0e9eb94668832d1208ad3b00a9791ef181eca4c/packages/completer-extension/src/index.ts#L198-L213
            const cell = this.widget.content.activeCell;
            const connector = new completion_1.LSPConnector({
                editor: cell.editor,
                connection: this.connection,
                coordinates_transform: (position) => this.notebook_as_editor.transform_to_notebook(cell, position),
                session: this.widget.session
            });
            const handler = this.completion_manager.register({
                connector,
                editor: cell.editor,
                parent: this.widget
            });
            this.widget.content.activeCellChanged.connect((notebook, cell) => {
                const connector = new completion_1.LSPConnector({
                    editor: cell.editor,
                    connection: this.connection,
                    coordinates_transform: (position) => this.notebook_as_editor.transform_to_notebook(cell, position),
                    session: this.widget.session
                });
                handler.editor = cell.editor;
                handler.connector = connector;
            });
        });
    }
    get_notebook_content() {
        return this.notebook_as_editor.getValue();
    }
    get cm_editor() {
        return this.notebook_as_editor;
    }
    get_document_content() {
        return this.get_notebook_content();
    }
    refresh_lsp_notebook_image(slot) {
        // TODO this is fired too often currently, debounce!
        this.connection.sendChange();
    }
}
exports.NotebookAdapter = NotebookAdapter;
