/// <reference types="@types/codemirror" />
import { DataConnector } from '@jupyterlab/coreutils';
import { CompletionHandler, ContextConnector, CompletionConnector } from '@jupyterlab/completer';
import { CodeEditor } from '@jupyterlab/codeeditor';
import { LspWsConnection } from 'lsp-editor-adapter';
import CodeMirror = require('codemirror');
import { IClientSession } from '@jupyterlab/apputils';
/**
 * A LSP connector for completion handlers.
 */
export declare class LSPConnector extends DataConnector<CompletionHandler.IReply, void, CompletionHandler.IRequest> {
    private readonly _editor;
    private readonly _connection;
    private _completion_characters;
    private _context_connector;
    private _kernel_connector;
    private _kernel_and_context_connector;
    transform_coordinates: (position: CodeMirror.Position) => CodeMirror.Position;
    /**
     * Create a new LSP connector for completion requests.
     *
     * @param options - The instantiation options for the LSP connector.
     */
    constructor(options: LSPConnector.IOptions);
    readonly fallback_connector: ContextConnector | CompletionConnector;
    /**
     * Fetch completion requests.
     *
     * @param request - The completion request text and details.
     */
    fetch(request: CompletionHandler.IRequest): Promise<CompletionHandler.IReply>;
    hint(editor: CodeEditor.IEditor, connection: LspWsConnection, completion_characters: Array<string>): Promise<CompletionHandler.IReply>;
    private merge_replies;
}
/**
 * A namespace for LSP connector statics.
 */
export declare namespace LSPConnector {
    /**
     * The instantiation options for cell completion handlers.
     */
    interface IOptions {
        /**
         * The editor used by the LSP connector.
         */
        editor: CodeEditor.IEditor;
        /**
         * The connection used by the LSP connector.
         */
        connection: LspWsConnection;
        coordinates_transform: (position: CodeMirror.Position) => CodeMirror.Position;
        session?: IClientSession;
    }
}
