"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const notebook_1 = require("@jupyterlab/notebook");
const codemirror_1 = require("@jupyterlab/codemirror");
const fileeditor_1 = require("@jupyterlab/fileeditor");
const coreutils_1 = require("@jupyterlab/coreutils");
const docmanager_1 = require("@jupyterlab/docmanager");
const fileeditor_2 = require("@krassowski/jupyterlab_go_to_definition/lib/jumpers/fileeditor");
const notebook_2 = require("@krassowski/jupyterlab_go_to_definition/lib/jumpers/notebook");
require("codemirror/addon/hint/show-hint.css");
require("codemirror/addon/hint/show-hint");
require("../style/index.css");
require("lsp-editor-adapter/lib/codemirror-lsp.css");
const completer_1 = require("@jupyterlab/completer");
const rendermime_1 = require("@jupyterlab/rendermime");
const notebook_3 = require("./adapters/notebook");
const file_editor_1 = require("./adapters/file_editor");
const file_editor_adapters = new Map();
const notebook_adapters = new Map();
const lsp_commands = [
    {
        id: 'lsp_get_definition',
        execute: (connection, position) => connection.getDefinition(position),
        isEnabled: (connection) => connection.isDefinitionSupported(),
        label: 'Jump to definition'
    },
    {
        id: 'lsp_get_type_definition',
        execute: (connection, position) => connection.getTypeDefinition(position),
        isEnabled: (connection) => connection.isTypeDefinitionSupported(),
        label: 'Highlight type definition'
    },
    {
        id: 'lsp_get_references',
        execute: (connection, position) => connection.getReferences(position),
        isEnabled: (connection) => connection.isReferencesSupported(),
        label: 'Highlight references'
    }
];
function is_context_menu_over_token(adapter) {
    let docPosition = adapter.get_doc_position_from_context_menu();
    if (!docPosition) {
        return false;
    }
    let token = adapter.cm_editor.getTokenAt(docPosition);
    return token.string !== '';
}
/**
 * The plugin registration information.
 */
const plugin = {
    id: '@krassowski/jupyterlab-lsp:plugin',
    requires: [
        fileeditor_1.IEditorTracker,
        notebook_1.INotebookTracker,
        coreutils_1.ISettingRegistry,
        apputils_1.ICommandPalette,
        docmanager_1.IDocumentManager,
        completer_1.ICompletionManager,
        rendermime_1.IRenderMimeRegistry
    ],
    activate: (app, fileEditorTracker, notebookTracker, settingRegistry, palette, documentManager, completion_manager, rendermime_registry) => {
        fileEditorTracker.widgetUpdated.connect((sender, widget) => {
            console.log(sender);
            console.log(widget);
            // TODO?
            // adapter.remove();
            // connection.close();
        });
        fileEditorTracker.widgetAdded.connect((sender, widget) => {
            let fileEditor = widget.content;
            if (fileEditor.editor instanceof codemirror_1.CodeMirrorEditor) {
                let jumper = new fileeditor_2.FileEditorJumper(widget, documentManager);
                let adapter = new file_editor_1.FileEditorAdapter(widget, jumper, app, completion_manager, rendermime_registry);
                file_editor_adapters.set(fileEditor.id, adapter);
            }
        });
        let is_context_menu_over_file_editor_token = () => {
            let fileEditor = fileEditorTracker.currentWidget.content;
            let adapter = file_editor_adapters.get(fileEditor.id);
            return is_context_menu_over_token(adapter);
        };
        for (let cmd of lsp_commands) {
            app.commands.addCommand(cmd.id, {
                execute: () => {
                    let fileEditor = fileEditorTracker.currentWidget.content;
                    let adapter = file_editor_adapters.get(fileEditor.id);
                    let docPosition = adapter.get_doc_position_from_context_menu();
                    cmd.execute(adapter.connection, docPosition);
                },
                isEnabled: is_context_menu_over_file_editor_token,
                isVisible: () => {
                    let fileEditor = fileEditorTracker.currentWidget.content;
                    let adapter = file_editor_adapters.get(fileEditor.id);
                    return (adapter && adapter.connection && cmd.isEnabled(adapter.connection));
                },
                label: cmd.label
            });
            app.contextMenu.addItem({
                selector: '.jp-FileEditor',
                command: cmd.id
            });
        }
        notebookTracker.widgetAdded.connect((sender, widget) => {
            // NOTE: assuming that the default cells content factory produces CodeMirror editors(!)
            let jumper = new notebook_2.NotebookJumper(widget, documentManager);
            let adapter = new notebook_3.NotebookAdapter(widget, jumper, app, completion_manager, rendermime_registry);
            notebook_adapters.set(widget.id, adapter);
        });
        // TODO de-duplicate commands creation, use some kind of an interface or factory
        let is_context_menu_over_notebook_token = () => {
            let notebook = notebookTracker.currentWidget;
            let adapter = notebook_adapters.get(notebook.id);
            return is_context_menu_over_token(adapter);
        };
        // position context menu entries after 10th but before 11th default entry
        // this lets it be before "Clear outputs" which is the last entry of the
        // CodeCell contextmenu and plays nicely with the first notebook entry
        // ('Clear all outputs') thus should stay as the last one.
        // see https://github.com/blink1073/jupyterlab/blob/3592afd328116a588e3307b4cdd9bcabc7fe92bb/packages/notebook-extension/src/index.ts#L802
        // TODO: PR bumping rank of clear all outputs instead?
        app.contextMenu.addItem({
            type: 'separator',
            selector: '.jp-Notebook .jp-CodeCell',
            rank: 10 + 1 / (lsp_commands.length + 2)
        });
        let i = 1;
        for (let cmd of lsp_commands) {
            i += 1;
            app.commands.addCommand(cmd.id + '_notebook', {
                execute: () => {
                    let notebook = notebookTracker.currentWidget;
                    let adapter = notebook_adapters.get(notebook.id);
                    let docPosition = adapter.get_doc_position_from_context_menu();
                    cmd.execute(adapter.connection, docPosition);
                },
                isEnabled: is_context_menu_over_notebook_token,
                isVisible: () => {
                    let notebook = notebookTracker.currentWidget;
                    let adapter = notebook_adapters.get(notebook.id);
                    return (adapter && adapter.connection && cmd.isEnabled(adapter.connection));
                },
                label: cmd.label
            });
            app.contextMenu.addItem({
                selector: '.jp-Notebook .jp-CodeCell',
                command: cmd.id + '_notebook',
                rank: 10 + i / (lsp_commands.length + 2)
            });
        }
        function updateOptions(settings) {
            // let options = settings.composite;
            // Object.keys(options).forEach((key) => {
            //  if (key === 'modifier') {
            //    // let modifier = options[key] as KeyModifier;
            //    CodeMirrorExtension.modifierKey = modifier;
            //  }
            // });
        }
        settingRegistry
            .load(plugin.id)
            .then(settings => {
            updateOptions(settings);
            settings.changed.connect(() => {
                updateOptions(settings);
            });
        })
            .catch((reason) => {
            console.error(reason.message);
        });
        /*
        // Add an application command
        const cmdIds = {
          // in future add more commands
          jumpNotebook: 'lsp:notebook-jump',
          jumpFileEditor: 'lsp:file-editor-jump'
        };
    
        // Add the command to the palette.
        palette.addItem({
          command: cmdIds.jumpNotebook,
          category: 'Notebook Cell Operations'
        });
        palette.addItem({
          command: cmdIds.jumpFileEditor,
          category: 'Text Editor'
        });
    
        function isEnabled(tracker: any) {
          return (): boolean =>
            tracker.currentWidget !== null &&
            tracker.currentWidget === app.shell.currentWidget;
        }
    
        app.commands.addCommand(cmdIds.jumpNotebook, {
          label: 'Jump to definition',
          execute: () => {
            let notebook_widget = notebookTracker.currentWidget;
            let notebook = notebook_widget.content;
    
            let jumper = new NotebookJumper(notebook_widget, documentManager);
            let cell = notebook_widget.content.activeCell;
            let editor = cell.editor;
    
            let position = editor.getCursorPosition();
            let token = editor.getTokenForPosition(position);
    
            jumper.jump_to_definition(
              { token, origin: null },
              notebook.activeCellIndex
            );
          },
          isEnabled: isEnabled(notebookTracker)
        });
    
        app.commands.addCommand(cmdIds.jumpFileEditor, {
          label: 'Jump to definition',
          execute: () => {
            let fileEditorWidget = fileEditorTracker.currentWidget;
            let fileEditor = fileEditorWidget.content;
    
            let jumper = new FileEditorJumper(fileEditorWidget, documentManager);
            let editor = fileEditor.editor;
    
            let position = editor.getCursorPosition();
            let token = editor.getTokenForPosition(position);
    
            jumper.jump_to_definition({ token, origin: null });
          },
          isEnabled: isEnabled(fileEditorTracker)
        });
    
        const bindings = [
          {
            selector: '.jp-Notebook.jp-mod-editMode',
            keys: ['Ctrl Alt B'],
            command: cmdIds.jumpNotebook
          },
          {
            selector: '.jp-FileEditor',
            keys: ['Ctrl Alt B'],
            command: cmdIds.jumpFileEditor
          }
        ];
    
        bindings.map(binding => app.commands.addKeyBinding(binding));
        */
    },
    autoStart: true
};
/**
 * Export the plugin as default.
 */
exports.default = plugin;
