/// <reference types="@types/codemirror/codemirror-showhint" />
/// <reference types="@types/codemirror" />
/// <reference types="codemirror/searchcursor" />
import { Notebook, NotebookPanel } from '@jupyterlab/notebook';
import { Cell } from '@jupyterlab/cells';
import CodeMirror = require('codemirror');
import { ShowHintOptions } from 'codemirror';
import { CodeEditor } from '@jupyterlab/codeeditor';
interface ICellTransform {
    cell: Cell;
    editor: CodeMirror.Editor;
    line_shift: number;
}
export declare class NotebookAsSingleEditor implements CodeMirror.Editor {
    notebook: Notebook;
    notebook_panel: NotebookPanel;
    line_cell_map: Map<number, ICellTransform>;
    cell_line_map: Map<Cell, number>;
    cm_editor_to_ieditor: Map<CodeMirror.Editor, CodeEditor.IEditor>;
    line_filter: string;
    private filtered_out_line_replacement;
    constructor(notebook_panel: NotebookPanel, line_filter?: string, filtered_out_line_replacement?: string);
    showHint: (options: ShowHintOptions) => void;
    state: any;
    addKeyMap(map: string | CodeMirror.KeyMap, bottom?: boolean): void;
    addLineClass(line: any, where: string, _class_: string): CodeMirror.LineHandle;
    addLineWidget(line: any, node: HTMLElement, options?: CodeMirror.LineWidgetOptions): CodeMirror.LineWidget;
    addOverlay(mode: any, options?: any): void;
    addPanel(node: HTMLElement, options?: CodeMirror.ShowPanelOptions): CodeMirror.Panel;
    addWidget(pos: CodeMirror.Position, node: HTMLElement, scrollIntoView: boolean): void;
    blockComment(from: Position, to: Position, options?: CodeMirror.CommentOptions): void;
    charCoords(pos: CodeMirror.Position, mode?: 'window' | 'page' | 'local'): {
        left: number;
        right: number;
        top: number;
        bottom: number;
    };
    clearGutter(gutterID: string): void;
    coordsChar(object: {
        left: number;
        top: number;
    }, mode?: 'window' | 'page' | 'local'): CodeMirror.Position;
    transform_to_notebook(cell: Cell, position: CodeMirror.Position): CodeMirror.Position;
    cursorCoords(where?: boolean, mode?: 'window' | 'page' | 'local'): {
        left: number;
        top: number;
        bottom: number;
    };
    cursorCoords(where?: CodeMirror.Position | null, mode?: 'window' | 'page' | 'local'): {
        left: number;
        top: number;
        bottom: number;
    };
    readonly any_editor: CodeMirror.Editor;
    defaultCharWidth(): number;
    defaultTextHeight(): number;
    endOperation(): void;
    execCommand(name: string): void;
    findPosH(start: CodeMirror.Position, amount: number, unit: string, visually: boolean): {
        line: number;
        ch: number;
        hitSide?: boolean;
    };
    findPosV(start: CodeMirror.Position, amount: number, unit: string): {
        line: number;
        ch: number;
        hitSide?: boolean;
    };
    findWordAt(pos: CodeMirror.Position): CodeMirror.Range;
    focus(): void;
    getDoc(): CodeMirror.Doc;
    getGutterElement(): HTMLElement;
    getInputField(): HTMLTextAreaElement;
    getLineTokens(line: number, precise?: boolean): CodeMirror.Token[];
    getModeAt(pos: CodeMirror.Position): any;
    getOption(option: string): any;
    getScrollInfo(): CodeMirror.ScrollInfo;
    getScrollerElement(): HTMLElement;
    getStateAfter(line?: number): any;
    getTokenAt(pos: CodeMirror.Position, precise?: boolean): CodeMirror.Token;
    getTokenTypeAt(pos: CodeMirror.Position): string;
    get_editor_at(pos: CodeMirror.Position): CodeMirror.Editor;
    get_cell_at(pos: CodeMirror.Position): Cell;
    transform(pos: CodeMirror.Position): CodeMirror.Position;
    getValue(seperator?: string): string;
    getViewport(): {
        from: number;
        to: number;
    };
    getWrapperElement(): HTMLElement;
    hasFocus(): boolean;
    heightAtLine(line: any, mode?: 'window' | 'page' | 'local', includeWidgets?: boolean): number;
    indentLine(line: number, dir?: string): void;
    isReadOnly(): boolean;
    lineAtHeight(height: number, mode?: 'window' | 'page' | 'local'): number;
    lineComment(from: Position, to: Position, options?: CodeMirror.CommentOptions): void;
    lineInfo(line: any): {
        line: any;
        handle: any;
        text: string;
        gutterMarkers: any;
        textClass: string;
        bgClass: string;
        wrapClass: string;
        widgets: any;
    };
    off(eventName: string, handler: (instance: CodeMirror.Editor) => void): void;
    off(eventName: 'change', handler: (instance: CodeMirror.Editor, change: CodeMirror.EditorChangeLinkedList) => void): void;
    off(eventName: 'changes', handler: (instance: CodeMirror.Editor, change: CodeMirror.EditorChangeLinkedList[]) => void): void;
    off(eventName: 'beforeChange', handler: (instance: CodeMirror.Editor, change: CodeMirror.EditorChangeCancellable) => void): void;
    off(eventName: 'cursorActivity', handler: (instance: CodeMirror.Editor) => void): void;
    off(eventName: 'beforeSelectionChange', handler: (instance: CodeMirror.Editor, selection: {
        head: CodeMirror.Position;
        anchor: CodeMirror.Position;
    }) => void): void;
    off(eventName: 'viewportChange', handler: (instance: CodeMirror.Editor, from: number, to: number) => void): void;
    off(eventName: 'gutterClick', handler: (instance: CodeMirror.Editor, line: number, gutter: string, clickEvent: Event) => void): void;
    off(eventName: 'focus', handler: (instance: CodeMirror.Editor) => void): void;
    off(eventName: 'blur', handler: (instance: CodeMirror.Editor) => void): void;
    off(eventName: 'scroll', handler: (instance: CodeMirror.Editor) => void): void;
    off(eventName: 'update', handler: (instance: CodeMirror.Editor) => void): void;
    off(eventName: 'renderLine', handler: (instance: CodeMirror.Editor, line: CodeMirror.LineHandle, element: HTMLElement) => void): void;
    off(eventName: 'mousedown' | 'dblclick' | 'touchstart' | 'contextmenu' | 'keydown' | 'keypress' | 'keyup' | 'cut' | 'copy' | 'paste' | 'dragstart' | 'dragenter' | 'dragover' | 'dragleave' | 'drop', handler: (instance: CodeMirror.Editor, event: Event) => void): void;
    off(eventName: string, handler: (doc: CodeMirror.Doc, event: any) => void): void;
    on(eventName: string, handler: (instance: CodeMirror.Editor) => void): void;
    on(eventName: 'change', handler: (instance: CodeMirror.Editor, change: CodeMirror.EditorChangeLinkedList) => void): void;
    on(eventName: 'changes', handler: (instance: CodeMirror.Editor, change: CodeMirror.EditorChangeLinkedList[]) => void): void;
    on(eventName: 'beforeChange', handler: (instance: CodeMirror.Editor, change: CodeMirror.EditorChangeCancellable) => void): void;
    on(eventName: 'cursorActivity', handler: (instance: CodeMirror.Editor) => void): void;
    on(eventName: 'beforeSelectionChange', handler: (instance: CodeMirror.Editor, selection: {
        head: CodeMirror.Position;
        anchor: CodeMirror.Position;
    }) => void): void;
    on(eventName: 'viewportChange', handler: (instance: CodeMirror.Editor, from: number, to: number) => void): void;
    on(eventName: 'gutterClick', handler: (instance: CodeMirror.Editor, line: number, gutter: string, clickEvent: Event) => void): void;
    on(eventName: 'focus', handler: (instance: CodeMirror.Editor) => void): void;
    on(eventName: 'blur', handler: (instance: CodeMirror.Editor) => void): void;
    on(eventName: 'scroll', handler: (instance: CodeMirror.Editor) => void): void;
    on(eventName: 'update', handler: (instance: CodeMirror.Editor) => void): void;
    on(eventName: 'renderLine', handler: (instance: CodeMirror.Editor, line: CodeMirror.LineHandle, element: HTMLElement) => void): void;
    on(eventName: 'mousedown' | 'dblclick' | 'touchstart' | 'contextmenu' | 'keydown' | 'keypress' | 'keyup' | 'cut' | 'copy' | 'paste' | 'dragstart' | 'dragenter' | 'dragover' | 'dragleave' | 'drop', handler: (instance: CodeMirror.Editor, event: Event) => void): void;
    on(eventName: 'overwriteToggle', handler: (instance: CodeMirror.Editor, overwrite: boolean) => void): void;
    on(eventName: string, handler: (doc: CodeMirror.Doc, event: any) => void): void;
    operation<T>(fn: () => T): T;
    refresh(): void;
    removeKeyMap(map: string | CodeMirror.KeyMap): void;
    removeLineClass(line: any, where: string, class_?: string): CodeMirror.LineHandle;
    removeOverlay(mode: any): void;
    scrollIntoView(pos: CodeMirror.Position | null, margin?: number): void;
    scrollIntoView(pos: {
        left: number;
        top: number;
        right: number;
        bottom: number;
    }, margin?: number): void;
    scrollIntoView(pos: {
        line: number;
        ch: number;
    }, margin?: number): void;
    scrollIntoView(pos: {
        from: CodeMirror.Position;
        to: CodeMirror.Position;
    }, margin?: number): void;
    scrollTo(x?: number | null, y?: number | null): void;
    setGutterMarker(line: any, gutterID: string, value: HTMLElement | null): CodeMirror.LineHandle;
    setOption(option: string, value: any): void;
    setSize(width: any, height: any): void;
    setValue(content: string): void;
    startOperation(): void;
    swapDoc(doc: CodeMirror.Doc): CodeMirror.Doc;
    toggleComment(options?: CodeMirror.CommentOptions): void;
    toggleOverwrite(value?: boolean): void;
    triggerOnKeyDown(event: Event): void;
    uncomment(from: Position, to: Position, options?: CodeMirror.CommentOptions): boolean;
}
export {};
