"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// @ts-ignore
class DocDispatcher {
    constructor(notebook_map) {
        // TODO
        this.notebook_map = notebook_map;
    }
    markText(from, to, options) {
        // TODO: edgecase: from and to in different cells
        let editor = this.notebook_map.get_editor_at(from);
        return editor
            .getDoc()
            .markText(this.transform(from), this.transform(to), options);
    }
    transform(position) {
        return this.notebook_map.transform(position);
    }
    getValue(seperator) {
        return this.notebook_map.getValue();
    }
    getCursor(start) {
        let cell = this.notebook_map.notebook.activeCell;
        let active_editor = cell.editor;
        let cursor = active_editor.editor.getDoc().getCursor(start);
        return this.notebook_map.transform_to_notebook(cell, cursor);
    }
}
// TODO: use proxy, as in https://stackoverflow.com/questions/51865430/typescript-compiler-does-not-know-about-es6-proxy-trap-on-class ?
// TODO: make all un-implemented methods issue a warning to a console for improved diagnostics
//  or just remove them altogether and suppress with ts-ignore so it will be more obvious what is implemented and what is not
//  ideally there would be some kind of a construct which defines a subset of CodeMirror.Editor (Interface?) needed for the adapter
class NotebookAsSingleEditor {
    constructor(notebook_panel, line_filter = '', filtered_out_line_replacement = '') {
        this.notebook_panel = notebook_panel;
        this.notebook = notebook_panel.content;
        this.line_cell_map = new Map();
        this.cell_line_map = new Map();
        this.cm_editor_to_ieditor = new Map();
        this.line_filter = line_filter;
        this.filtered_out_line_replacement = filtered_out_line_replacement;
    }
    addKeyMap(map, bottom) { }
    addLineClass(line, where, _class_) {
        return undefined;
    }
    addLineWidget(line, node, options) {
        return undefined;
    }
    addOverlay(mode, options) {
        for (let cell of this.notebook.widgets) {
            // TODO: use some more intelligent strategy to determine editors to test
            let cm_editor = cell.editor;
            cm_editor.editor.addOverlay(mode, options);
        }
    }
    addPanel(node, 
    // @ts-ignore
    options
    // @ts-ignore
    ) {
        return undefined;
    }
    addWidget(pos, node, scrollIntoView) { }
    blockComment(from, to, 
    // @ts-ignore
    options) { }
    charCoords(pos, mode) {
        try {
            let editor = this.get_editor_at(pos);
            return editor.charCoords(pos, mode);
        }
        catch (e) {
            console.log(e);
            return { bottom: 0, left: 0, right: 0, top: 0 };
        }
    }
    clearGutter(gutterID) { }
    coordsChar(object, mode) {
        for (let cell of this.notebook.widgets) {
            // TODO: use some more intelligent strategy to determine editors to test
            let cm_editor = cell.editor;
            let pos = cm_editor.editor.coordsChar(object, mode);
            // @ts-ignore
            if (pos.outside === true) {
                continue;
            }
            return this.transform_to_notebook(cell, pos);
        }
    }
    transform_to_notebook(cell, position) {
        // TODO: if cell is not known, refresh
        let shift = this.cell_line_map.get(cell);
        if (shift === undefined) {
            throw Error('Cell not found in cell_line_map');
        }
        return Object.assign({}, position, { line: position.line + shift });
    }
    cursorCoords(where, mode) {
        if (typeof where !== 'boolean') {
            let editor = this.get_editor_at(where);
            return editor.cursorCoords(this.transform(where));
        }
        return { bottom: 0, left: 0, top: 0 };
    }
    get any_editor() {
        return this.notebook.widgets[0].editor.editor;
    }
    defaultCharWidth() {
        return this.any_editor.defaultCharWidth();
    }
    defaultTextHeight() {
        return this.any_editor.defaultTextHeight();
    }
    endOperation() {
        for (let cell of this.notebook.widgets) {
            let cm_editor = cell.editor;
            cm_editor.editor.endOperation();
        }
    }
    execCommand(name) {
        for (let cell of this.notebook.widgets) {
            let cm_editor = cell.editor;
            cm_editor.editor.execCommand(name);
        }
    }
    findPosH(start, amount, unit, visually) {
        return { ch: 0, line: 0 };
    }
    findPosV(start, amount, unit) {
        return { ch: 0, line: 0 };
    }
    findWordAt(pos) {
        return undefined;
    }
    focus() { }
    getDoc() {
        let dummy_doc = new DocDispatcher(this);
        // @ts-ignore
        return dummy_doc;
    }
    getGutterElement() {
        return undefined;
    }
    getInputField() {
        return undefined;
    }
    getLineTokens(line, precise) {
        return [];
    }
    getModeAt(pos) {
        return this.get_editor_at(pos).getModeAt(this.transform(pos));
    }
    getOption(option) {
        return this.any_editor.getOption(option);
    }
    getScrollInfo() {
        return undefined;
    }
    getScrollerElement() {
        return undefined;
    }
    getStateAfter(line) { }
    getTokenAt(pos, precise) {
        if (pos === undefined) {
            return;
        }
        let editor = this.get_editor_at(pos);
        return editor.getTokenAt(this.transform(pos));
    }
    getTokenTypeAt(pos) {
        let editor = this.get_editor_at(pos);
        return editor.getTokenTypeAt(this.transform(pos));
    }
    // TODO: make a mapper class, with mapping function only
    get_editor_at(pos) {
        return this.line_cell_map.get(pos.line).editor;
    }
    get_cell_at(pos) {
        return this.line_cell_map.get(pos.line).cell;
    }
    transform(pos) {
        return Object.assign({}, pos, { 
            // DOES it work like that? TODO
            line: pos.line - this.line_cell_map.get(pos.line).line_shift });
    }
    getValue(seperator) {
        let value;
        this.line_cell_map.clear();
        this.cell_line_map.clear();
        this.cm_editor_to_ieditor.clear();
        // TODO: make this configurable
        let empty_lines_between_cells = 2;
        let lines_padding = '\n'.repeat(empty_lines_between_cells);
        let last_line = 0;
        let all_lines = [];
        this.notebook.widgets.every(cell => {
            let codemirror_editor = cell.editor;
            let cm_editor = codemirror_editor.editor;
            this.cm_editor_to_ieditor.set(cm_editor, cell.editor);
            if (cell.model.type == 'code') {
                let lines = cm_editor.getValue(seperator);
                // TODO: use blacklist for cells with foreign language code
                //  TODO: have a second LSP for the foreign language!
                if (lines.startsWith('%%')) {
                    return true;
                }
                // one empty line is necessary to separate code blocks, next 'n' lines are to silence linters
                // and the final cell does not get the additional lines (thanks to the use of join, see below)
                let filtered_lines = new Array();
                let lines_array = lines.split('\n');
                this.cell_line_map.set(cell, last_line);
                for (let i = 0; i < lines_array.length; i++) {
                    // TODO: use better structure (tree with ranges?)
                    this.line_cell_map.set(last_line + i, {
                        editor: cm_editor,
                        line_shift: last_line,
                        cell
                    });
                    if (!this.line_filter ||
                        lines_array[i].match(this.line_filter) === null) {
                        filtered_lines.push(lines_array[i]);
                    }
                    else {
                        filtered_lines.push(this.filtered_out_line_replacement);
                    }
                }
                all_lines.push(filtered_lines.join('\n') + '\n');
                // note filtered_lines.length === lines_array.length
                last_line += filtered_lines.length + empty_lines_between_cells;
            }
            return true;
        });
        value = all_lines.join(lines_padding);
        return value;
    }
    getViewport() {
        return { from: 0, to: 0 };
    }
    getWrapperElement() {
        return this.notebook_panel.node;
    }
    hasFocus() {
        return false;
    }
    heightAtLine(line, mode, includeWidgets) {
        return 0;
    }
    indentLine(line, dir) { }
    isReadOnly() {
        return false;
    }
    lineAtHeight(height, mode) {
        return 0;
    }
    lineComment(from, to, 
    // @ts-ignore
    options) { }
    lineInfo(line) {
        return {
            bgClass: '',
            gutterMarkers: undefined,
            handle: undefined,
            line: undefined,
            text: '',
            textClass: '',
            widgets: undefined,
            wrapClass: ''
        };
    }
    off(eventName, handler) { }
    on(eventName, handler) {
        let wrapped_handler = (instance_or_doc, a, b, c) => {
            let editor = instance_or_doc;
            try {
                editor.getDoc();
                // @ts-ignore
                return handler(this, a, b, c);
            }
            catch (e) {
                // TODO verify that the error was due to getDoc not existing on editor
                console.log(e);
                // also this is not currently in use
                console.log('Dispatching wrapped doc handler with', this);
                // @ts-ignore
                return handler(this.getDoc(), a, b, c);
            }
        };
        const cells_with_handlers = new Set();
        for (let cell of this.notebook.widgets) {
            // TODO: use some more intelligent strategy to determine editors to test
            let cm_editor = cell.editor.editor;
            cells_with_handlers.add(cell);
            // @ts-ignore
            cm_editor.on(eventName, wrapped_handler);
        }
        this.notebook.activeCellChanged.connect((notebook, cell) => {
            let cm_editor = cell.editor.editor;
            if (!cells_with_handlers.has(cell)) {
                // @ts-ignore
                cm_editor.on(eventName, wrapped_handler);
            }
        });
    }
    operation(fn) {
        return undefined;
    }
    refresh() { }
    removeKeyMap(map) { }
    removeLineClass(line, where, class_) {
        return undefined;
    }
    removeOverlay(mode) { }
    scrollIntoView(pos, margin) { }
    scrollTo(x, y) { }
    setGutterMarker(line, gutterID, value) {
        return undefined;
    }
    setOption(option, value) { }
    setSize(width, height) { }
    setValue(content) { }
    startOperation() { }
    swapDoc(doc) {
        return undefined;
    }
    // @ts-ignore
    toggleComment(options) { }
    toggleOverwrite(value) { }
    triggerOnKeyDown(event) { }
    uncomment(from, to, 
    // @ts-ignore
    options) {
        return false;
    }
}
exports.NotebookAsSingleEditor = NotebookAsSingleEditor;
